==> mcss-0.0.31.tar.gz: mcss-0.0.31/mcss/Psystem.h <==
// p-system class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#ifndef PSYSTEM_H
#define PSYSTEM_H

#include <sbml/SBMLTypes.h>
#include <sstream>
#include <string>
#include <stdexcept>
#include <ecsb/ecsb.h>

#include "Rule.h"
#include "RuleSet.h"
#include "SimulationParameters.h"
#include "SimulationDatasets.h"
#include "SimulationAlgorithm.h"
#include "Lattice.h"
#include "MulticompartmentGillespie.h"
#include "MulticompartmentGillespieGD.h"
#include "MulticompartmentGillespieQ.h"
#include "MulticompartmentGillespieCellularPotts.h"
#include "LogarithmicDirectMethod.h"
#include "Alphabet.h"
#include "Compartment.h"

using namespace ECSB;

namespace Simulation {

// p-system class
class Psystem {
		// model id
		string id;
		
		// compartments
		unsigned long num_compartments;  // total number of compartments
		class Simulation::Compartment **compartment;  // compartment

		// alphabet of objects
		unsigned long num_species;  // total number of species
		class Alphabet *alphabet;

		// rules
		unsigned long num_rulesets;  // total number of rule templates
		class RuleSet **ruleset;  // rule templates

		// simulation algorithm
		class SimulationAlgorithm *simalg;  // simulation algorithm

		// initial compartments
		unsigned long num_initial_compartments;
		class Simulation::Compartment **initial_compartment;

		// spatial lattice
		Lattice *lattice;
		unsigned long xdim;  // x dimension
		unsigned long ydim;  // y dimension
		unsigned long zdim;  // z dimension

		// initial spatial lattice
		unsigned long initial_xdim;
		unsigned long initial_ydim;
		unsigned long initial_zdim;

		// simulation flags
		bool log_reactions;  // log reaction data
		bool log_volumes;  // log volume data

		// lattice periodic boundary condition flags
		bool periodic_x, periodic_y, periodic_z;

		// duplicate initial amounts for all templates
		bool duplicate_initial_amounts;
		
		// keep levels of degraded species
		bool keep_degraded;

		// volume growth type
		int growth_type;

		// set default values for initialised class
		void setInitialisationValues(void);
	public:
		// constructors
		Psystem();
		Psystem(SimulationParameters &);
		Psystem(char *);

		// destructor
		~Psystem();

		// general access methods
		void setId(std::string);
		std::string getId(void);		
		unsigned long getNumCompartments(void);
		Alphabet *getAlphabet(void);
		SimulationAlgorithm *getSimulationAlgorithm(void);
		void setSimulationAlgorithm(SimulationAlgorithm *);
		void setSimulationAlgorithm(string name, unsigned long seed = 0);
		void logReactions(bool);
		void show(void);
		Simulation::Compartment *getCompartment(unsigned long);
		Lattice *getLattice(void);
		unsigned long getLatticeX(void);
		unsigned long getLatticeY(void);
		unsigned long getLatticeZ(void);
		Simulation::Rule *getRule(unsigned long, unsigned long, unsigned long);
		unsigned long getNumSpecies(void);
		unsigned long getNumRuleSets(void);
		unsigned long getNumRuleSets(unsigned long);
		Simulation::RuleSet *getRuleSet(unsigned long);
		unsigned long getRuleSetIndex(string);
		unsigned long getRuleSetIndex(unsigned long, unsigned long);
		unsigned long getNumRules(unsigned long, unsigned long);
		unsigned long getSpeciesIndex(string);
		bool isDegradation(string);
		unsigned long getNumTimepoints(void);
		unsigned int ***getAmountsDataset(void);
		double getCompartmentVolume(unsigned long);
		void setCompartmentVolume(unsigned long c, double v);
		void setReactionConstant(unsigned long c, unsigned long rs, \
				unsigned long r, double v);
		double getReactionConstant(unsigned long c, unsigned long rs, \
				unsigned long r);
		void resetSimulation(void);
		bool isChannel(unsigned long, unsigned long, unsigned long);
		string getTargetName(unsigned long, unsigned long, unsigned long);
		double getLogInterval(void);
		string getCompartmentName(unsigned long);
		double getSimulationTime(void);
		
		// create p system from sbml model
		void createFromSBML(SimulationParameters &);

		// create p system from sbml model
		void createFromSBML(char *);

		// create p system from xml model
		void createFromXML(SimulationParameters &);

		// create alphabet from xml model
		void createAlphabet(std::vector<std::string>);

		// create label ids from xml model
		std::map<std::string,unsigned long> createLabelsId( \
				std::map<std::string, ECSB::CAPRuleSet>);

		// create rulesets from xml model
		void createRuleSets(std::map<std::string,ECSB::CAPRuleSet>, \
				std::map<std::string, unsigned long>);

		// create compartments from xml model
		void createCompartments(ECSB::CAPSystem model, std::map<std::string, \
				ECSB::InitialMultiset> combinedInitialMultisets);

		// create alphabet from sbml model
		void createAlphabet(Model *);

		// create compartments from sbml model
		void createCompartments(Model *);

		// add a duplicate compartment
		Simulation::Compartment *addDuplicateCompartment(unsigned long, \
				unsigned long *, const char);

		// return index of compartment given id
		unsigned long getCompartmentIndex(std::string);

		// return index of compartment given position
		unsigned long getCompartmentIndex(unsigned long *);

		// calculate target compartment of rule
		unsigned long calculateRuleTarget(unsigned long, unsigned long, \
				unsigned long);

		// execute a specified rule
		void executeRule(unsigned long, unsigned long, unsigned long);

		// initialise p-system simulation
		int preprocess(void);

		// perform one iteration of p-system
		int execute(void);

		// perform one iteration of specified p-system compartment
		void execute(unsigned long);

		// iterate p-system for specified time and runs
		void execute(double const max_time);

		// save model as psys xml file
		void outputXML(char *);

		// get levels of species in compartment
		unsigned long getSpeciesLevel(unsigned long s, unsigned long c);

		// calculate if a rule is valid for a compartment
		bool isValidRule(unsigned long, unsigned long, unsigned long);

		// return true if compartment defines template
		bool isTemplateCompartment(string);
		
		// test if compartment name is valid
		bool validateCompartmentName(string);

		// test if rule name is valid
		bool validateRuleName(string);

		// test if rule structure is valid
		bool validateRuleStructure(Reaction *);

		// validate distribution-based reaction constant
		void validateReactionDistribution(string);
		
		// validate species name
		void validateSpeciesName(string);
		
		// get compartment position from SBML
		unsigned long *getCompartmentPositionFromSBML(string);

		// get compartment template indices from SBML
		void getCompartmentTemplatesFromSBML(string, unsigned long *);

		// get number of compartment templates from SBML
		unsigned int getNumCompartmentTemplatesFromSBML(string);

		// get name of compartment from SBML
		string getCompartmentNameFromSBML(string);

		// get index of ruleset from name
		unsigned long getSBMLIndex(string);

		// get generation time from SBML
		double getGenerationTimeFromSBML(string);
		
		// save model as stochkit problem definition
		void outputStochKit(SimulationParameters &, char *);

		// save list of initial compartments
		void saveInitialCompartments(void);

		// reset compartments to initial state
		void resetCompartments(void);

		// set valid ruleset rules for compartment
		void setValidRules(unsigned long);

		// check if string valid SMBML SId
		bool isValidSBMLSId(string);

		// write a fully compliant sbml model
		void outputSBML(char *);

		// dump model to binary format file
		void dump(char *);

		// add a new compartment
		unsigned long addCompartment(string name, unsigned long *pos = NULL);

		// add a species
		unsigned long addSpecies(string name);

		// add a ruleset
		unsigned long addRuleSet(string name);
		
		// add a rule to a ruleset
		unsigned long addRule(unsigned long rsid, string name, \
				unsigned long num_reactants, unsigned long *reactants, \
				unsigned long num_products, unsigned long *products, \
				double reaction_constant);

		// set rule to be a transportation rule
		void setChannel(unsigned long rsid, unsigned long rid);
				
		// add a ruleset to a compartment
		void addRuleSetToCompartment(unsigned long rsid, unsigned long cid);

		// set target for translocation rule
		void setRuleTarget(unsigned long cid, unsigned long rsid, \
				unsigned long rid, unsigned long target_cid);

		// set level of species in compartment
		void setSpeciesLevel(unsigned long cid, unsigned long sid, \
				unsigned long level);

		// clone rule from ruleset in compartment
		unsigned long cloneRule(unsigned long cid, unsigned long rsid, \
				unsigned long rid);

		// set initial levels given in templates modules
		void setInitialLevels(void);

		// get rule vector from SBML
		bool getRuleVectorFromSBML(string n, long int *target);
		
		// get rule target from SBML
		string getRuleTargetFromSBML(string n);
		
		// get rule target type from SBML
		string getRuleTargetTypeFromSBML(string n);

		// get parent compartment name from SBML
		string getParentCompartmentFromSBML(string n);
};

}

#endif

==> mcss-0.0.35.tar.gz: mcss-0.0.35/mcss/Psystem.h <==
// p-system class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#ifndef PSYSTEM_H
#define PSYSTEM_H

#include <sbml/SBMLTypes.h>
#include <sstream>
#include <string>
#include <stdexcept>
#include <ecsb/ecsb.h>

#include "Rule.h"
#include "RuleSet.h"
#include "SimulationParameters.h"
#include "SimulationDatasets.h"
#include "SimulationAlgorithm.h"
#include "Lattice.h"
#include "MulticompartmentGillespie.h"
#include "MulticompartmentGillespieGD.h"
#include "MulticompartmentGillespieQ.h"
#include "MulticompartmentGillespieQ2.h"
#include "MulticompartmentGillespieQG.h"
#include "MulticompartmentGillespieQ2G.h"
#include "MulticompartmentGillespieCellularPotts.h"
#include "LogarithmicDirectMethod.h"
#include "Alphabet.h"
#include "Compartment.h"

using namespace ECSB;

namespace Simulation {

// p-system class
class Psystem {
		// model id
		string id;

		// compartments
		unsigned long num_compartments;  // total number of compartments
		class Simulation::Compartment **compartment;  // compartment

		// alphabet of objects
		unsigned long num_species;  // total number of species
		class Alphabet *alphabet;

		// rules
		unsigned long num_rulesets;  // total number of rule templates
		class RuleSet **ruleset;  // rule templates

		// simulation algorithm
		class SimulationAlgorithm *simalg;  // simulation algorithm

		// initial compartments
		unsigned long num_initial_compartments;
		class Simulation::Compartment **initial_compartment;

		// spatial lattice
		Lattice *lattice;
		unsigned long xdim;  // x dimension
		unsigned long ydim;  // y dimension
		unsigned long zdim;  // z dimension

		// initial spatial lattice
		unsigned long initial_xdim;
		unsigned long initial_ydim;
		unsigned long initial_zdim;

		// simulation flags
		bool log_reactions;  // log reaction data
		bool log_volumes;  // log volume data

		// lattice periodic boundary condition flags
		bool periodic_x, periodic_y, periodic_z;

		// duplicate initial amounts for all templates
		bool duplicate_initial_amounts;

		// keep levels of degraded species
		bool keep_degraded;

		// volume growth type
		int growth_type;

		// set default values for initialised class
		void setInitialisationValues(void);
	public:
		// constructors
		Psystem();
		Psystem(SimulationParameters &);
		Psystem(char *);

		// destructor
		~Psystem();

		// general access methods
		void setId(std::string);
		std::string getId(void);
		unsigned long getNumCompartments(void);
		Alphabet *getAlphabet(void);
		SimulationAlgorithm *getSimulationAlgorithm(void);
		void setSimulationAlgorithm(SimulationAlgorithm *);
		void setSimulationAlgorithm(string name, unsigned long seed = 0);
		void logReactions(bool);
		void show(void);
		Simulation::Compartment *getCompartment(unsigned long);
		Lattice *getLattice(void);
		unsigned long getLatticeX(void);
		unsigned long getLatticeY(void);
		unsigned long getLatticeZ(void);
		Simulation::Rule *getRule(unsigned long, unsigned long, unsigned long);
		unsigned long getNumSpecies(void);
		unsigned long getNumRuleSets(void);
		unsigned long getNumRuleSets(unsigned long);
		Simulation::RuleSet *getRuleSet(unsigned long);
		unsigned long getRuleSetIndex(string);
		unsigned long getRuleSetIndex(unsigned long, unsigned long);
		unsigned long getNumRules(unsigned long, unsigned long);
		unsigned long getSpeciesIndex(string);
		bool isDegradation(string);
		unsigned long getNumTimepoints(void);
		unsigned int ***getAmountsDataset(void);
		double getCompartmentVolume(unsigned long);
		void setCompartmentVolume(unsigned long c, double v);
		void setReactionConstant(unsigned long c, unsigned long rs, \
				unsigned long r, double v);
		double getReactionConstant(unsigned long c, unsigned long rs, \
				unsigned long r);
		void resetSimulation(void);
		bool isChannel(unsigned long, unsigned long, unsigned long);
		string getTargetName(unsigned long, unsigned long, unsigned long);
		double getLogInterval(void);
		string getCompartmentName(unsigned long);
		double getSimulationTime(void);

		// create p system from sbml model
		void createFromSBML(SimulationParameters &);

		// create p system from sbml model
		void createFromSBML(char *);

		// create p system from xml model
		void createFromXML(SimulationParameters &);

		// create alphabet from xml model
		void createAlphabet(std::vector<std::string>);

		// create label ids from xml model
		std::map<std::string,unsigned long> createLabelsId( \
				std::map<std::string, ECSB::LPPRuleSet>);

		// create rulesets from xml model
		void createRuleSets(std::map<std::string,ECSB::LPPRuleSet>, \
				std::map<std::string, unsigned long>);

		// create compartments from xml model
		void createCompartments(ECSB::LPPsystem model, std::map<std::string, \
				ECSB::InitialMultiset> combinedInitialMultisets);

		// create alphabet from sbml model
		void createAlphabet(Model *);

		// create compartments from sbml model
		void createCompartments(Model *);

		// add a duplicate compartment
		Simulation::Compartment *addDuplicateCompartment(unsigned long, \
				unsigned long *, const char);

		// return index of compartment given id
		unsigned long getCompartmentIndex(std::string);

		// return index of compartment given position
		unsigned long getCompartmentIndex(unsigned long *);

		// calculate target compartment of rule
		unsigned long calculateRuleTarget(unsigned long, unsigned long, \
				unsigned long);

		// execute a specified rule
		void executeRule(unsigned long, unsigned long, unsigned long);

		// initialise p-system simulation
		int preprocess(void);

		// perform one iteration of p-system
		int execute(void);

		// perform one iteration of specified p-system compartment
		void execute(unsigned long);

		// iterate p-system for specified time and runs
		void execute(double const max_time);

		// save model as psys xml file
		void outputXML(char *);

		// get levels of species in compartment
		unsigned long getSpeciesLevel(unsigned long s, unsigned long c);

		// calculate if a rule is valid for a compartment
		bool isValidRule(unsigned long, unsigned long, unsigned long);

		// return true if compartment defines template
		bool isTemplateCompartment(string);

		// test if compartment name is valid
		bool validateCompartmentName(string);

		// test if rule name is valid
		bool validateRuleName(string);

		// test if rule structure is valid
		bool validateRuleStructure(Reaction *);

		// validate distribution-based reaction constant
		void validateReactionDistribution(string);

		// validate species name
		void validateSpeciesName(string);

		// get compartment position from SBML
		unsigned long *getCompartmentPositionFromSBML(string);

		// get compartment template indices from SBML
		void getCompartmentTemplatesFromSBML(string, unsigned long *);

		// get number of compartment templates from SBML
		unsigned int getNumCompartmentTemplatesFromSBML(string);

		// get name of compartment from SBML
		string getCompartmentNameFromSBML(string);

		// get index of ruleset from name
		unsigned long getSBMLIndex(string);

		// get generation time from SBML
		double getGenerationTimeFromSBML(string);

		// save model as stochkit problem definition
		void outputStochKit(SimulationParameters &, char *);

		// save list of initial compartments
		void saveInitialCompartments(void);

		// reset compartments to initial state
		void resetCompartments(void);

		// set valid ruleset rules for compartment
		void setValidRules(unsigned long);

		// check if string valid SMBML SId
		bool isValidSBMLSId(string);

		// write a fully compliant sbml model
		void outputSBML(char *);

		// dump model to binary format file
		void dump(char *);

		// add a new compartment
		unsigned long addCompartment(string name, unsigned long *pos = NULL);

		// add a species
		unsigned long addSpecies(string name);

		// add a ruleset
		unsigned long addRuleSet(string name);

		// add a rule to a ruleset
		unsigned long addRule(unsigned long rsid, string name, \
				unsigned long num_reactants, unsigned long *reactants, \
				unsigned long num_products, unsigned long *products, \
				double reaction_constant);

		// set rule to be a transportation rule
		void setChannel(unsigned long rsid, unsigned long rid);

		// add a ruleset to a compartment
		void addRuleSetToCompartment(unsigned long rsid, unsigned long cid);

		// set target for translocation rule
		void setRuleTarget(unsigned long cid, unsigned long rsid, \
				unsigned long rid, unsigned long target_cid);

		// set level of species in compartment
		void setSpeciesLevel(unsigned long cid, unsigned long sid, \
				unsigned long level);

		// clone rule from ruleset in compartment
		unsigned long cloneRule(unsigned long cid, unsigned long rsid, \
				unsigned long rid);

		// set initial levels given in templates modules
		void setInitialLevels(void);

		// get rule vector from SBML
		bool getRuleVectorFromSBML(string n, long int *target);

		// get rule target from SBML
		string getRuleTargetFromSBML(string n);

		// get rule target type from SBML
		string getRuleTargetTypeFromSBML(string n);

		// get parent compartment name from SBML
		string getParentCompartmentFromSBML(string n);

		// create + populate cellular potts lattice
		void createLattice(SimulationParameters &);
};

}

#endif
